from __future__ import annotations

import sys
from pathlib import Path
from subprocess import CalledProcessError
from unittest.mock import MagicMock, patch

from git import Commit

from bar_raiser.checks.annotate_pyright import (
    get_annotations_and_actions_for_pyright_check,
    main,
)

REPO_DIR = "/home/user/bar_raiser"
WORKING_DIR = "/home/user/bar_raiser/subfolder"

PYRIGHT_OUTPUT_WITH_ERROR = """
{"version": "1.1.304",
    "time": "1705431974306",
    "generalDiagnostics": [
        {"file": "/home/user/bar_raiser/subfolder/checks/annotate_pyright.py",
            "severity": "error",
            "message": "Try statement must have at least one except or finally clause",
            "range": {"start": {"line": 15,
                    "character": 4
                },
                "end": {"line": 15,
                    "character": 7
                }
            }
        }
    ],
    "summary": {"filesAnalyzed": 45,
        "errorCount": 1,
        "warningCount": 0,
        "informationCount": 0,
        "timeInSec": 4.554
    }
}
"""

PYRIGHT_OUTPUT_WITH_UNNECESSARY_IGNORE = r"""
{"version": "1.1.304",
    "time": "1705455668409",
    "generalDiagnostics": [
        {"file": "/home/user/bar_raiser/subfolder/checks/annotate_pyright.py",
            "severity": "error",
            "message": "Unnecessary \"# pyright: ignore\" rule: \"reportUnknownMemberType\"",
            "range": {"start": {"line": 31,
                    "character": 52
                },
                "end": {"line": 31,
                    "character": 75
                }
            }
        }
    ],
    "summary": {"filesAnalyzed": 46,
        "errorCount": 1,
        "warningCount": 0,
        "informationCount": 0,
        "timeInSec": 4.654
    }
}
"""

PYRIGHT_OUTPUT_WITH_NO_ERROR = """
{
    "version": "1.1.304",
    "time": "1705456899041",
    "generalDiagnostics": [],
    "summary": {
        "filesAnalyzed": 46,
        "errorCount": 0,
        "warningCount": 0,
        "informationCount": 0,
        "timeInSec": 4.959
    }
}
"""

PYRIGHT_OUTPUT_WITH_RULE_ERROR = r"""
{
    "version": "1.1.304",
    "time": "1705530427151",
    "generalDiagnostics": [
        {
            "file": "/home/user/bar_raiser/subfolder/lib/common.py",
            "severity": "error",
            "message": "Type of \"line\" is unknown",
            "range": {
                "start": {
                    "line": 190,
                    "character": 12
                },
                "end": {
                    "line": 190,
                    "character": 16
                }
            },
            "rule": "reportUnknownVariableType"
        },
        {
            "file": "/home/user/bar_raiser/subfolder/lib/common.py",
            "severity": "error",
            "message": "Argument type is unknown\n  Argument corresponds to parameter \"__object\" in function \"append\"",
            "range": {
                "start": {
                    "line": 201,
                    "character": 20
                },
                "end": {
                    "line": 201,
                    "character": 24
                }
            },
            "rule": "reportUnknownArgumentType"
        }
    ],
    "summary": {
        "filesAnalyzed": 1,
        "errorCount": 10,
        "warningCount": 0,
        "informationCount": 0,
        "timeInSec": 0.691
    }
}
"""


def test_get_annotations_and_actions_for_pyright() -> None:
    with patch(
        "bar_raiser.checks.annotate_pyright.Path.cwd",
        return_value=Path(WORKING_DIR),
    ):
        assert get_annotations_and_actions_for_pyright_check(
            Path(REPO_DIR), PYRIGHT_OUTPUT_WITH_ERROR
        ) == (
            [
                {
                    "path": "subfolder/checks/annotate_pyright.py",
                    "start_line": 16,
                    "end_line": 16,
                    "annotation_level": "failure",
                    "message": "Try statement must have at least one except or finally clause",
                }
            ],
            [],
        )

        assert get_annotations_and_actions_for_pyright_check(
            Path(REPO_DIR), PYRIGHT_OUTPUT_WITH_UNNECESSARY_IGNORE
        ) == (
            [
                {
                    "path": "subfolder/checks/annotate_pyright.py",
                    "start_line": 32,
                    "end_line": 32,
                    "annotation_level": "failure",
                    "message": 'Unnecessary "# pyright: ignore" rule: "reportUnknownMemberType"',
                }
            ],
            [
                {
                    "label": "autofix",
                    "description": "Click to auto-push an autofix commit",
                    "identifier": "pyright-ignores",
                }
            ],
        )

        assert get_annotations_and_actions_for_pyright_check(
            Path(REPO_DIR), PYRIGHT_OUTPUT_WITH_NO_ERROR
        ) == ([], [])

        print(
            get_annotations_and_actions_for_pyright_check(
                Path(REPO_DIR), PYRIGHT_OUTPUT_WITH_RULE_ERROR
            )
        )
        assert get_annotations_and_actions_for_pyright_check(
            Path(REPO_DIR), PYRIGHT_OUTPUT_WITH_RULE_ERROR
        ) == (
            [
                {
                    "path": "subfolder/lib/common.py",
                    "start_line": 191,
                    "end_line": 191,
                    "annotation_level": "failure",
                    "message": 'Type of "line" is unknown [reportUnknownVariableType]',
                },
                {
                    "path": "subfolder/lib/common.py",
                    "start_line": 202,
                    "end_line": 202,
                    "annotation_level": "failure",
                    "message": 'Argument type is unknown\n  Argument corresponds to parameter "__object" in function "append" [reportUnknownArgumentType]',
                },
            ],
            [],
        )


def test_main() -> None:
    target_module = "bar_raiser.checks.annotate_pyright"

    with (
        patch(f"{target_module}.create_check_run") as mock_create_check_run,
        patch(f"{target_module}.get_github_repo"),
        patch(f"{target_module}.get_git_repo") as mock_git_repo,
        patch(f"{target_module}.exit") as mock_exit,
        patch(
            "bar_raiser.checks.annotate_pyright.Path.cwd",
            return_value=Path(WORKING_DIR),
        ),
        patch.object(sys, "argv", ["annotate_pyright.py"]),
        patch(f"{target_module}.get_head_sha", return_value="1"),
    ):
        mock_git_repo.return_value.working_dir = Path(REPO_DIR)
        mock_git_repo.return_value.head.commit.parents = [
            MagicMock(spec=Commit, hexsha="1"),
        ]
        with patch(
            f"{target_module}.check_output",
            side_effect=CalledProcessError(
                1, [], output=PYRIGHT_OUTPUT_WITH_ERROR.encode("utf-8")
            ),
        ):
            main()
            assert len(mock_create_check_run.call_args_list) == 1
            kwargs = mock_create_check_run.call_args_list[0].kwargs
            assert kwargs["name"] == "python-pyright-report"
            assert kwargs["head_sha"] == "1"
            assert kwargs["conclusion"] == "action_required"
            assert kwargs["title"] == "Python Pyright Type Checker"
            assert len(kwargs["annotations"]) == 1
            assert len(kwargs["actions"]) == 0
            mock_exit.assert_called_once_with(1)
            mock_create_check_run.reset_mock()
            mock_exit.reset_mock()

        with patch(
            f"{target_module}.check_output",
            side_effect=CalledProcessError(
                0, [], output=PYRIGHT_OUTPUT_WITH_NO_ERROR.encode("utf-8")
            ),
        ):
            main()
            assert len(mock_create_check_run.call_args_list) == 1
            kwargs = mock_create_check_run.call_args_list[0].kwargs
            assert kwargs["name"] == "python-pyright-report"
            assert kwargs["head_sha"] == "1"
            assert kwargs["conclusion"] == "success"
            assert kwargs["title"] == "Python Pyright Type Checker"
            assert len(kwargs["annotations"]) == 0
            assert len(kwargs["actions"]) == 0
            mock_exit.assert_called_once_with(0)



from __future__ import annotations

from json import dumps
from pathlib import Path
from unittest.mock import MagicMock, patch

import pytest
from github.PullRequest import PullRequest
from github.Team import Team as GithubTeam

from bar_raiser.autofixes.notify_reviewer_teams import (
    LABEL_TO_REMOVE,
    ReviewRequest,
    create_slack_message,
    main,
    process_pull_request,
    process_review_request,
)


@pytest.fixture
def mock_pull_request() -> PullRequest:
    mock = MagicMock(spec=PullRequest)
    mock.user.login = "testuser"
    mock.html_url = "https://github.com/test/pr/1"
    mock.number = 1
    mock.title = "Test PR"
    return mock


@pytest.fixture
def mock_team() -> GithubTeam:
    mock = MagicMock(spec=GithubTeam)
    mock.organization.login = "Greenbax"
    mock.slug = "test-team"
    return mock


def test_create_slack_message(mock_pull_request: PullRequest) -> None:
    review_request = ReviewRequest(
        team="@Greenbax/test-team",
        channel="test-channel",
        slack_id="U123",
        pull_request=mock_pull_request,
    )
    message = create_slack_message(review_request)
    assert "U123" in message
    assert "PR-1" in message
    assert "Test PR" in message
    assert "test-team" in message


@patch(
    "bar_raiser.autofixes.notify_reviewer_teams.get_slack_user_icon_url_and_username"
)
@patch("bar_raiser.autofixes.notify_reviewer_teams.post_a_slack_message")
def test_process_review_request_success(
    mock_post_message: MagicMock,
    mock_get_user_info: MagicMock,
    mock_team: GithubTeam,
    mock_pull_request: PullRequest,
) -> None:
    mock_get_user_info.return_value = ("icon_url", "username")

    with patch(
        "pathlib.Path.read_text",
        return_value=dumps({"@Greenbax/test-team": "test-channel"}),
    ):
        comment, success = process_review_request(
            mock_team,
            mock_pull_request,
            "U123",
            dry_run="test-channel",
            github_team_to_slack_channels_path=Path("test-path"),
            github_team_to_slack_channels_help_msg="",
        )
    assert success
    assert "test-channel" in comment
    mock_post_message.assert_called_once()


@patch(
    "bar_raiser.autofixes.notify_reviewer_teams.get_slack_user_icon_url_and_username"
)
@patch("bar_raiser.autofixes.notify_reviewer_teams.post_a_slack_message")
def test_process_review_request_none_channel(
    mock_post_message: MagicMock,
    mock_get_user_info: MagicMock,
    mock_team: GithubTeam,
    mock_pull_request: PullRequest,
) -> None:
    mock_get_user_info.return_value = ("icon_url", "username")

    with patch(
        "pathlib.Path.read_text",
        return_value=dumps({}),
    ):
        comment, success = process_review_request(
            mock_team,
            mock_pull_request,
            "U123",
            dry_run="test-channel",
            github_team_to_slack_channels_path=Path("test-path"),
            github_team_to_slack_channels_help_msg="",
        )
    assert not success
    assert "Slack channel not found" in comment
    mock_post_message.assert_not_called()


@patch(
    "bar_raiser.autofixes.notify_reviewer_teams.get_slack_user_icon_url_and_username"
)
@patch("bar_raiser.autofixes.notify_reviewer_teams.post_a_slack_message")
def test_process_review_request_dry_run(
    mock_post_message: MagicMock,
    mock_get_user_info: MagicMock,
    mock_team: GithubTeam,
    mock_pull_request: PullRequest,
) -> None:
    mock_get_user_info.return_value = ("icon_url", "username")

    with patch(
        "pathlib.Path.read_text",
        return_value=dumps({"@Greenbax/test-team": "test-channel"}),
    ):
        comment, success = process_review_request(
            mock_team,
            mock_pull_request,
            "U123",
            dry_run="test-channel",
            github_team_to_slack_channels_path=Path("test-path"),
            github_team_to_slack_channels_help_msg="",
        )
    assert success
    assert "test-team" in comment
    mock_post_message.assert_called_once()


@patch("bar_raiser.autofixes.notify_reviewer_teams.get_slack_channel_from_mapping_path")
def test_process_pull_request_no_slack_id(
    mock_get_slack_id: MagicMock,
    mock_pull_request: PullRequest,
) -> None:
    mock_get_slack_id.return_value = None

    comment = process_pull_request(
        mock_pull_request,
        dry_run="test-channel",
        github_login_to_slack_ids_path=Path("test-path-1"),
        github_login_to_slack_ids_help_msg="Please update the mapping in `test-path-1`",
        github_team_to_slack_channels_path=Path("test-path-2"),
        github_team_to_slack_channels_help_msg="Please update the mapping in `test-path-2`",
        only_notify_team_slug=None,
    )
    assert (
        comment
        == "No author slack_id found for author testuser.\nPlease update the mapping in `test-path-1`\n"
    )


@patch("bar_raiser.autofixes.notify_reviewer_teams.get_slack_channel_from_mapping_path")
@patch("bar_raiser.autofixes.notify_reviewer_teams.process_review_request")
def test_process_pull_request_success(
    mock_process_request: MagicMock,
    mock_get_slack_id: MagicMock,
    mock_pull_request: PullRequest,
    mock_team: GithubTeam,
) -> None:
    mock_get_slack_id.return_value = "U123"
    mock_process_request.return_value = ("Test comment", True)

    mock_pull_request.get_review_requests = MagicMock(return_value=[[mock_team]])

    comment = process_pull_request(
        mock_pull_request,
        dry_run="test-channel",
        github_login_to_slack_ids_path=Path("test-path-1"),
        github_login_to_slack_ids_help_msg="",
        github_team_to_slack_channels_path=Path("test-path-2"),
        github_team_to_slack_channels_help_msg="",
        only_notify_team_slug=None,
    )
    assert comment == "Test comment"
    mock_process_request.assert_called_once()


@pytest.mark.parametrize(
    (
        "only_notify_team_slug_arg",
        "requested_teams_slugs",
        "process_review_request_return_value",
        "expected_comment",
        "expected_calls_to_process_review_request",
    ),
    [
        (
            "target-team",  # Test 1: Target team is a reviewer
            ["target-team", "other-team"],
            ("Comment for target-team", True),
            "Comment for target-team",
            1,
        ),
        (
            "non-existent-team",  # Test 2: Target team is NOT a reviewer
            ["team-a", "team-b"],
            ("Should not be called", True),
            "",
            0,
        ),
        (
            None,  # Test 3: only_notify_team_slug is None (all teams)
            ["team-1", "team-2"],
            ("Comment for team", True),
            "Comment for teamComment for team",  # Called for both teams
            2,
        ),
        (
            "target-team-no-channel",  # Test 4: Target team is reviewer, but process_review_request returns empty (e.g. no channel)
            ["target-team-no-channel", "another-team"],
            (
                "",
                False,
            ),  # Simulates an issue like a missing Slack channel mapping for the target team
            "",
            1,
        ),
        (
            "target-team",  # Test 5: Target team is a reviewer, no other reviewers
            ["target-team"],
            ("Comment for target-team", True),
            "Comment for target-team",
            1,
        ),
        (
            "not-a-reviewer",  # Test 6: Target team specified, but PR has no reviewers
            [],
            ("Should not be called", False),
            "",
            0,
        ),
        (
            None,  # Test 7: No target team, no reviewers
            [],
            ("Should not be called", False),
            "No team review requests found.",  # Updated expected message based on recent changes
            0,
        ),
    ],
)
@patch("bar_raiser.autofixes.notify_reviewer_teams.get_slack_channel_from_mapping_path")
@patch("bar_raiser.autofixes.notify_reviewer_teams.process_review_request")
def test_process_pull_request_only_notify_team(  # noqa: PLR0917
    mock_process_review_request: MagicMock,
    mock_get_author_slack_id: MagicMock,
    mock_pull_request: PullRequest,
    only_notify_team_slug_arg: str | None,
    requested_teams_slugs: list[str],
    process_review_request_return_value: tuple[str, bool],
    expected_comment: str,
    expected_calls_to_process_review_request: int,
) -> None:
    mock_get_author_slack_id.return_value = "U_AUTHOR123"  # For author lookup
    mock_process_review_request.return_value = process_review_request_return_value

    # Setup mock teams based on requested_teams_slugs
    mock_teams: list[MagicMock] = []
    for slug in requested_teams_slugs:
        team = MagicMock(spec=GithubTeam)
        team.organization.login = "Greenbax"
        team.slug = slug
        mock_teams.append(team)

    mock_pull_request.get_review_requests = MagicMock(return_value=[mock_teams])

    comment = process_pull_request(
        pull_request=mock_pull_request,
        dry_run="",  # Dry run channel not relevant for this focused test
        github_login_to_slack_ids_path=Path("dummy_login_to_slack.json"),
        github_login_to_slack_ids_help_msg="login help",
        github_team_to_slack_channels_path=Path("dummy_team_to_channels.json"),
        github_team_to_slack_channels_help_msg="team help",
        only_notify_team_slug=only_notify_team_slug_arg,
    )

    assert comment == expected_comment
    assert (
        mock_process_review_request.call_count
        == expected_calls_to_process_review_request
    )

    if expected_calls_to_process_review_request == 1 and only_notify_team_slug_arg:
        # Check that process_review_request was called with the correct target team
        called_team_arg = mock_process_review_request.call_args[0][0]
        assert called_team_arg.slug == only_notify_team_slug_arg


@patch("bar_raiser.autofixes.notify_reviewer_teams.get_pull_request")
def test_main(mock_get_pull_request: MagicMock) -> None:
    mock_get_pull_request.return_value = None
    with pytest.raises(SystemExit) as exc:
        main()
    assert exc.value.code == 2

    mock_pull_request = MagicMock(spec=PullRequest)
    mock_pull_request.labels = []
    mock_pull_request.draft = False
    mock_get_pull_request.return_value = mock_pull_request

    mock_pull_request.labels = [MagicMock(name=LABEL_TO_REMOVE)]
    with (
        patch(
            "bar_raiser.autofixes.notify_reviewer_teams.process_pull_request",
            return_value="Test comment",
        ),
        patch(
            "sys.argv",
            [
                "notify_reviewer_teams.py",
                "github_login_to_slack_ids.json",
                "github_login_to_slack_ids_help_msg",
                "github_team_to_slack_channels.json",
                "github_team_to_slack_channels_help_msg",
            ],
        ),
    ):
        main()
    mock_pull_request.remove_from_labels.assert_called_once_with(LABEL_TO_REMOVE)



from __future__ import annotations

import sys
from pathlib import Path
from subprocess import CalledProcessError
from unittest.mock import MagicMock, patch

from bar_raiser.checks.annotate_ruff import (
    RUFF_CHECK_PATTERNS,
    RUFF_FORMAT_PATTERNS,
    main,
)
from bar_raiser.utils.check import get_annotations_and_actions
from bar_raiser.utils.github import Autofixes

RUFF_FORMAT_OUTPUT = """\
error: Failed to format apply_autofixes.py: source contains syntax errors: ParseError { error: UnrecognizedToken(Colon, None), offset: 554, source_path: "<filename>" }
error: Failed to format test_annotate_ruff.py: source contains syntax errors: ParseError { error: UnrecognizedToken(Semi, None), offset: 957, source_path: "<filename>" }
Would reformat: annotate_ruff.py
1 file would be reformatted, 7975 files left unchanged
"""

RUFF_CHECK_OUTPUT = """\
error: Failed to parse annotate_ruff.py:68:14: Unexpected token 'if'
charts.py:1:1: I002 [*] Missing required import: `from __future__ import annotations`
test_annotate_ruff.py:5:22: F401 [*] `unittest.mock` imported but unused
Found 2 errors.
[*] 2 fixable with the `--fix` option.
"""

REPO_DIR = "/home/user/bar_raiser"
WORKING_DIR = "/home/user/bar_raiser/subfolder"


def test_get_annotations_and_actions_for_ruff_format() -> None:
    with patch(
        "bar_raiser.checks.annotate_ruff.Path.cwd",
        return_value=Path(WORKING_DIR),
    ):
        assert get_annotations_and_actions(
            Path(REPO_DIR), RUFF_FORMAT_OUTPUT, RUFF_FORMAT_PATTERNS, Autofixes.RUFF
        ) == (
            [
                {
                    "path": "subfolder/apply_autofixes.py",
                    "start_line": 554,
                    "end_line": 554,
                    "annotation_level": "failure",
                    "message": "source contains syntax errors: ParseError { error: UnrecognizedToken(Colon, None)",
                },
                {
                    "path": "subfolder/test_annotate_ruff.py",
                    "start_line": 957,
                    "end_line": 957,
                    "annotation_level": "failure",
                    "message": "source contains syntax errors: ParseError { error: UnrecognizedToken(Semi, None)",
                },
                {
                    "path": "subfolder/annotate_ruff.py",
                    "start_line": 1,
                    "end_line": 1,
                    "annotation_level": "failure",
                    "message": "Ruff would reformat this file.",
                },
            ],
            {
                "label": "autofix",
                "description": "Click to auto-push an autofix commit",
                "identifier": "autofix-ruff",
            },
        )


def test_get_annotations_and_actions_for_ruff_check() -> None:
    with patch(
        "bar_raiser.checks.annotate_ruff.Path.cwd",
        return_value=Path(WORKING_DIR),
    ):
        assert get_annotations_and_actions(
            Path(REPO_DIR), RUFF_CHECK_OUTPUT, RUFF_CHECK_PATTERNS, Autofixes.RUFF
        ) == (
            [
                {
                    "path": "subfolder/annotate_ruff.py",
                    "start_line": 68,
                    "end_line": 68,
                    "annotation_level": "failure",
                    "message": "Unexpected token 'if'",
                },
                {
                    "path": "subfolder/charts.py",
                    "start_line": 1,
                    "end_line": 1,
                    "annotation_level": "failure",
                    "message": "I002 [*] Missing required import: `from __future__ import annotations`",
                },
                {
                    "path": "subfolder/test_annotate_ruff.py",
                    "start_line": 5,
                    "end_line": 5,
                    "annotation_level": "failure",
                    "message": "F401 [*] `unittest.mock` imported but unused",
                },
            ],
            {
                "label": "autofix",
                "description": "Click to auto-push an autofix commit",
                "identifier": "autofix-ruff",
            },
        )


def test_main() -> None:
    returncode = -1
    target_module = "bar_raiser.checks.annotate_ruff"

    def mock_check_output(cmds: list[str], stderr: int):
        match cmds[1]:
            case "format":
                raise CalledProcessError(
                    returncode, [], output=RUFF_FORMAT_OUTPUT.encode("utf-8")
                )
            case "check":
                raise CalledProcessError(
                    returncode, [], output=RUFF_CHECK_OUTPUT.encode("utf-8")
                )

    with (
        patch(f"{target_module}.create_check_run") as mock_create_check_run,
        patch(
            f"{target_module}.check_output",
            mock_check_output,
        ),
        patch(f"{target_module}.get_github_repo"),
        patch(f"{target_module}.get_git_repo") as mock_git_repo,
        patch(f"{target_module}.get_head_sha", return_value="1"),
        patch(f"{target_module}.exit") as mock_exit,
        patch(
            "bar_raiser.checks.annotate_ruff.Path.cwd",
            return_value=Path(WORKING_DIR),
        ),
        patch.object(sys, "argv", ["annotate_ruff.py"]),
    ):
        mock_git_repo.return_value.working_dir = Path(WORKING_DIR)
        main()
        assert len(mock_create_check_run.call_args_list) == 1
        kwargs = mock_create_check_run.call_args_list[0].kwargs
        assert kwargs["name"] == "python-ruff-report"
        assert kwargs["head_sha"] == "1"
        assert kwargs["conclusion"] == "action_required"
        assert kwargs["title"] == "Python Ruff formatter and linter"
        assert len(kwargs["annotations"]) == 6
        assert len(kwargs["actions"]) == 1
        mock_exit.assert_called_once_with(returncode)


def test_main_only_fail_on_format() -> None:
    returncode = -1
    target_module = "bar_raiser.checks.annotate_ruff"

    def mock_check_output(cmds: list[str], stderr: int) -> MagicMock | None:
        match cmds[1]:
            case "format":
                return MagicMock()
            case "check":
                raise CalledProcessError(
                    returncode, [], output=RUFF_CHECK_OUTPUT.encode("utf-8")
                )

    with (
        patch(f"{target_module}.create_check_run") as mock_create_check_run,
        patch(
            f"{target_module}.check_output",
            mock_check_output,
        ),
        patch(f"{target_module}.get_github_repo"),
        patch(f"{target_module}.get_git_repo") as mock_git_repo,
        patch(f"{target_module}.get_head_sha", return_value="1"),
        patch(f"{target_module}.exit") as mock_exit,
        patch(
            "bar_raiser.checks.annotate_ruff.Path.cwd",
            return_value=Path(WORKING_DIR),
        ),
        patch.object(sys, "argv", ["annotate_ruff.py"]),
    ):
        mock_git_repo.return_value.working_dir = Path(REPO_DIR)
        main()
        assert len(mock_create_check_run.call_args_list) == 1
        kwargs = mock_create_check_run.call_args_list[0].kwargs
        assert kwargs["name"] == "python-ruff-report"
        assert kwargs["head_sha"] == "1"
        assert kwargs["conclusion"] == "action_required"
        assert kwargs["title"] == "Python Ruff formatter and linter"
        assert len(kwargs["annotations"]) == 3
        assert len(kwargs["actions"]) == 1
        mock_exit.assert_called_once_with(returncode)



from __future__ import annotations

import os
from datetime import datetime
from enum import StrEnum
from logging import INFO, basicConfig, getLogger
from os import chdir, environ
from pathlib import Path
from subprocess import check_output
from sys import stdout
from typing import TYPE_CHECKING, Literal, TypedDict
from zoneinfo import ZoneInfo

from git.repo import Repo
from github import Github, GithubIntegration, InputGitTreeElement
from github.CheckRun import CheckRun

if TYPE_CHECKING:
    from github.PullRequest import PullRequest
    from github.Repository import Repository

ANNOTATION_PAGE_SIZE = 50

logger = getLogger(__name__)


def get_github() -> Github:
    integration = GithubIntegration(environ["APP_ID"], environ["PRIVATE_KEY"])
    owner = environ["GITHUB_REPOSITORY_OWNER"]
    short_repo = environ["GITHUB_REPOSITORY"][len(owner) + 1 :]
    install = integration.get_installation(owner, short_repo)
    token = integration.get_access_token(install.id).token
    return Github(token)


def get_github_repo() -> Repository:
    github = get_github()
    logger.info(github.get_rate_limit())
    repo_name = environ["GITHUB_REPOSITORY"]
    return github.get_repo(repo_name)


def get_pull_request() -> PullRequest | None:
    try:
        return get_github_repo().get_pull(int(environ["PULL_NUMBER"]))
    except Exception:
        return None


def get_head_sha() -> str:
    if get_pull_request():  # pull requests have a virtual merge commit
        return get_git_repo().head.commit.parents[-1].hexsha
    return get_git_repo().head.commit.hexsha


def has_previous_issue_comment(pull: PullRequest, author: str, body: str) -> bool:
    for comment in pull.get_issue_comments():
        if comment.user.login == author and body in comment.body:
            return True
    return False


def get_git_repo() -> Repo:
    return Repo(".", search_parent_directories=True)


class Annotation(TypedDict):
    path: str
    start_line: int
    end_line: int
    annotation_level: str
    message: str


class Action(TypedDict):
    label: str
    description: str
    identifier: str


def create_check_run(
    *,
    repo: Repository,
    name: str,
    head_sha: str,
    conclusion: Literal["action_required", "success", "failure"],
    title: str,
    summary: str,
    annotations: list[Annotation],
    actions: list[Action],
) -> list[CheckRun]:
    checks = list[CheckRun]()
    while True:
        batch, annotations = (
            annotations[:ANNOTATION_PAGE_SIZE],
            annotations[ANNOTATION_PAGE_SIZE:],
        )
        check = repo.create_check_run(
            name=name,
            head_sha=head_sha,
            conclusion=conclusion,
            output={  # pyright: ignore[reportArgumentType]
                "title": title,
                "summary": summary,
                "annotations": batch,
            },
            actions=actions,  # pyright: ignore[reportArgumentType]
        )
        logger.info(check.html_url)
        checks.append(check)
        if len(annotations) == 0:
            break
    return checks


def commit_changes(
    repo: Repository, branch: str, sha: str, paths: list[str], commit_message: str
) -> None:
    batch_size = 200
    num_batches = len(paths) // batch_size + 1
    count = 0
    cwd = os.getcwd()
    chdir(Path(__file__).parent.parent.parent.parent)
    while paths:
        elements: list[InputGitTreeElement] = []
        for path in paths[:batch_size]:
            content = open(path, encoding="utf-8").read() if Path(path).exists() else ""
            blob = repo.create_git_blob(content, "utf-8")
            elements.append(InputGitTreeElement(path, "100644", "blob", sha=blob.sha))
        existing_tree = repo.get_git_tree(sha)
        existing_commit = repo.get_git_commit(sha)
        new_tree = repo.create_git_tree(elements, existing_tree)
        message = (
            f"{commit_message} ({count + 1}/{num_batches})"
            if num_batches > 1
            else commit_message
        )
        new_commit = repo.create_git_commit(message, new_tree, [existing_commit])
        repo.get_git_ref(f"heads/{branch}").edit(new_commit.sha)
        sha = new_commit.sha
        paths = paths[batch_size:]
        count += 1
    chdir(cwd)


def get_updated_paths(pull: PullRequest) -> list[str]:
    return [
        file.filename
        for file in pull.get_files()
        if file.status != "removed" and file.filename.endswith(".py")
    ]


def run_codemod_and_commit_changes(
    github_repo: Repository,
    pull_number: int,
    commands: list[list[str]],
    commit_message: str,
    run_on_updated_paths: bool,
) -> None:
    pull = github_repo.get_pull(pull_number)
    if run_on_updated_paths:
        pull_updated_paths = get_updated_paths(pull)
        commands = [command + pull_updated_paths for command in commands]

    for command in commands:
        check_output(command)

    git_repo = get_git_repo()
    updated_paths: list[str] = [
        i.b_path  # pyright: ignore[reportUnknownMemberType]
        for i in git_repo.index.diff(  # pyright: ignore[reportUnknownMemberType,reportUnknownVariableType]
            None
        )
    ]
    if run_on_updated_paths:
        updated_paths = [path for path in updated_paths if path in updated_paths]

    logger.info(f"Updated paths: {updated_paths}")
    if len(updated_paths) == 0:
        msg = "No updated paths to commit."
        pull.create_issue_comment(msg)
        logger.info(msg)
    else:
        commit_changes(
            repo=github_repo,
            branch=pull.head.ref,
            sha=pull.head.sha,
            paths=updated_paths,
            commit_message=commit_message,
        )
        logger.info(f"Changes committed to {pull.html_url}.")


def create_a_pull_request(
    github_repo: Repository,
    codemod: str,
    actor: str,
    extra_body: str = "",
) -> None:
    git_repo = get_git_repo()
    codemod_name = (
        codemod.strip().split("codemod ")[1].split(" ")[0]
        if "codemod " in codemod
        else codemod.replace(" ", "-")
    )
    updated_paths: list[str] = [
        i.b_path  # pyright: ignore[reportUnknownMemberType]
        for i in git_repo.index.diff(  # pyright: ignore[reportUnknownMemberType,reportUnknownVariableType]
            None
        )
    ]
    if len(updated_paths) == 0:
        msg = "No updated paths to commit."
        logger.info(msg)
    else:
        logger.info(f"Updated paths: {updated_paths}")
        commit = github_repo.get_commit("master")
        time = datetime.now(ZoneInfo("US/Pacific")).strftime("%Y%m%d-%H-%M-%S")
        branch = f"codemod-{codemod_name}-{actor}-{time}"
        ref = f"refs/heads/{branch}"
        github_repo.create_git_ref(ref, commit.sha)
        title = f"Codemod {codemod_name} by {actor} at {time}"
        commit_changes(
            repo=github_repo,
            branch=branch,
            sha=commit.sha,
            paths=updated_paths,
            commit_message=title,
        )
        body = f"Codemod Command: `{codemod}`\nCreated from {environ['GITHUB_SERVER_URL']}/{environ['GITHUB_REPOSITORY']}/actions/runs/{environ['GITHUB_RUN_ID']} by {actor}"
        if extra_body:
            body += f"\n\n{extra_body}"
        pull = github_repo.create_pull(title=title, body=body, base="master", head=ref)
        pull.add_to_assignees(actor)
        pull.enable_automerge(merge_method="SQUASH")
        logger.info(f"Changes committed to {pull.html_url}")


def initialize_logging() -> None:
    basicConfig(
        stream=stdout,
        level=INFO,
        format="%(asctime)s %(levelname)s %(module)s - %(funcName)s: %(message)s",
    )


class Autofixes(StrEnum):
    RUFF = "autofix-ruff"
    PYRIGHT_IGNORES = "pyright-ignores"



from __future__ import annotations

import sys
from argparse import ArgumentParser
from dataclasses import dataclass
from logging import getLogger
from pathlib import Path
from typing import TYPE_CHECKING

from github.Team import Team

from bar_raiser.utils.slack import (
    get_slack_user_icon_url_and_username,
)

if TYPE_CHECKING:
    from github.PullRequest import PullRequest


from bar_raiser.utils.github import get_pull_request, initialize_logging
from bar_raiser.utils.slack import (
    get_slack_channel_from_mapping_path,
    post_a_slack_message,
)

logger = getLogger(__name__)


LABEL_TO_REMOVE = "autofix-notify-reviewer-teams"


@dataclass
class ReviewRequest:
    team: str
    channel: str | None
    slack_id: str
    pull_request: PullRequest


def create_slack_message(review_request: ReviewRequest) -> str:
    """Create a Slack message for a review request."""
    return (
        f"Hi team, Could we please get reviews on <@{review_request.slack_id}>'s "
        f"<{review_request.pull_request.html_url}|PR-{review_request.pull_request.number}> "
        f"({review_request.pull_request.title})? A review from the *{review_request.team.split('/')[-1]}* "
        "team is required. Thanks! 🙏"
    )


def process_review_request(  # noqa: PLR0917
    request: Team,
    pull_request: PullRequest,
    slack_id: str,
    dry_run: str,
    github_team_to_slack_channels_path: Path,
    github_team_to_slack_channels_help_msg: str,
) -> tuple[str, bool]:
    """Process a single review request and return the comment and success status."""
    team = f"@{request.organization.login}/{request.slug}"
    channel = get_slack_channel_from_mapping_path(
        team, github_team_to_slack_channels_path
    )

    if channel is None:
        error_msg = f"Slack channel not found for Github team: {team}\n{github_team_to_slack_channels_help_msg}\n"
        logger.error(error_msg)
        return (
            error_msg,
            False,
        )

    if dry_run:
        channel = dry_run

    if channel:
        review_request = ReviewRequest(
            team=team, channel=channel, slack_id=slack_id, pull_request=pull_request
        )
        message = create_slack_message(review_request)
        icon_url, username = get_slack_user_icon_url_and_username(slack_id)

        post_a_slack_message(
            channel=channel,
            text=message,
            icon_url=icon_url,
            username=username,
        )

        success_msg = f"Sent message to [Slack channel](https://try-evergreen.slack.com/archives/{channel}) for reviewer {team}.\n"
        logger.info(
            f"Sent message to https://try-evergreen.slack.com/archives/{channel} for reviewer {team}."
        )
        return success_msg, True

    return "", False


def process_pull_request(  # noqa: PLR0917
    pull_request: PullRequest,
    dry_run: str,
    github_login_to_slack_ids_path: Path,
    github_login_to_slack_ids_help_msg: str,
    github_team_to_slack_channels_path: Path,
    github_team_to_slack_channels_help_msg: str,
    only_notify_team_slug: str | None,
) -> str:
    """Process all review requests for a pull request."""
    author_login = pull_request.user.login
    slack_id = get_slack_channel_from_mapping_path(
        author_login, github_login_to_slack_ids_path
    )
    if slack_id is None:
        comment = f"No author slack_id found for author {author_login}.\n{github_login_to_slack_ids_help_msg}\n"
        logger.error(comment)
        return comment

    accumulated_comments = ""

    for team_requests_list in pull_request.get_review_requests():  # noqa: PLR1702
        for requested_team_obj in team_requests_list:
            if isinstance(requested_team_obj, Team):
                current_team_slug = requested_team_obj.slug
                current_team_full_name = (
                    f"@{requested_team_obj.organization.login}/{current_team_slug}"
                )

                if only_notify_team_slug:
                    if current_team_slug == only_notify_team_slug:
                        single_request_comment, _ = process_review_request(
                            requested_team_obj,
                            pull_request,
                            slack_id,
                            dry_run,
                            github_team_to_slack_channels_path,
                            github_team_to_slack_channels_help_msg,
                        )
                        if single_request_comment:
                            accumulated_comments += single_request_comment
                        # Only process this team if it's the target
                    else:
                        logger.info(
                            f"Skipping notification for team {current_team_full_name} as --only-notify-team is set to {only_notify_team_slug}."
                        )
                else:
                    # No specific team targeted, process all teams
                    single_request_comment, _ = process_review_request(
                        requested_team_obj,
                        pull_request,
                        slack_id,
                        dry_run,
                        github_team_to_slack_channels_path,
                        github_team_to_slack_channels_help_msg,
                    )
                    if single_request_comment:
                        accumulated_comments += single_request_comment

    if len(accumulated_comments) == 0 and not only_notify_team_slug:
        return "No team review requests found."
    return accumulated_comments


def main() -> None:
    parser = ArgumentParser(
        description="Run checks and optionally send Slack DMs on failure."
    )
    parser.add_argument(
        "--dry-run",
        type=str,
        help="Dry run the autofix and send a message to a test Slack channel. Provide a test channel ID.",
        default="",
    )
    parser.add_argument(
        "github_login_to_slack_ids",
        type=Path,
        help="Path to a JSON file containing a mapping from GitHub login to Slack IDs.",
        default=None,
    )
    parser.add_argument(
        "github_login_to_slack_ids_help_msg",
        type=str,
        help="A help message for updating the github_login_to_slack_ids mapping file.",
        default="",
    )
    parser.add_argument(
        "github_team_to_slack_channels",
        type=Path,
        help="Path to a JSON file containing a mapping from GitHub team to Slack channels.",
        default=None,
    )
    parser.add_argument(
        "github_team_to_slack_channels_help_msg",
        type=str,
        help="A help message for updating the github_team_to_slack_channels mapping file.",
        default="",
    )
    parser.add_argument(
        "--only-notify-team",
        type=str,
        help="Only send notification to the specific team slug if they are a requested reviewer.",
        default=None,
    )
    args = parser.parse_args()
    pull = get_pull_request()
    dry_run = args.dry_run
    logger.info(f"Dry run: {dry_run}")

    if pull is None:
        logger.error("No pull request found.")
        sys.exit(1)

    if pull.draft:
        comment = "Pull request is a draft and is not ready for review."
        logger.error(comment)
    else:
        comment = process_pull_request(
            pull,
            args.dry_run,
            args.github_login_to_slack_ids,
            args.github_login_to_slack_ids_help_msg,
            args.github_team_to_slack_channels,
            args.github_team_to_slack_channels_help_msg,
            args.only_notify_team,
        )

    if comment:
        pull.create_issue_comment(body=comment)
    pull.remove_from_labels(LABEL_TO_REMOVE)


if __name__ == "__main__":
    initialize_logging()
    main()



from __future__ import annotations

import argparse
from os import environ
from pathlib import Path
from unittest.mock import Mock, mock_open, patch

from bar_raiser.checks.annotate_diff_cover import (
    TIP_TEXT,
    DiffCoverJson,
    get_annotations,
    get_conclusion,
    get_github_repo,
    get_ranges,
    get_summary,
    main,
)
from bar_raiser.utils.github import Annotation


@patch.dict(
    environ,
    {
        "APP_ID": "_ID",
        "PRIVATE_KEY": "_KEY",
        "GITHUB_REPOSITORY_OWNER": "ZipHQ",
        "GITHUB_REPOSITORY": "ZipHQ/bar-raiser",
    },
)
def test_get_repo() -> None:
    with (
        patch("bar_raiser.utils.github.GithubIntegration"),
        patch("bar_raiser.utils.github.Github") as mock_github,
    ):
        get_github_repo()
        mock_github.return_value.get_repo.assert_called_with("ZipHQ/bar-raiser")


def test_get_ranges() -> None:
    assert get_ranges([1, 2, 3, 4, 5, 8, 9, 11, 13, 14, 15]) == [
        (1, 5),
        (8, 9),
        (11, 11),
        (13, 15),
    ]


def test_get_conclusion() -> None:
    assert get_conclusion(80) == "success"
    assert get_conclusion(75) == "success"
    assert get_conclusion(74) == "action_required"


def test_get_annotations() -> None:
    diff_cover_json: DiffCoverJson = {
        "src_stats": {
            "github/annotate_missing_test_coverage.py": {
                "violation_lines": [
                    14,
                    15,
                    16,
                    17,
                    18,
                    19,
                    20,
                    21,
                    62,
                    63,
                    64,
                    65,
                    66,
                    67,
                    76,
                    80,
                    90,
                ],
            }
        },
        "total_num_lines": 48,
        "total_num_violations": 17,
        "total_percent_covered": 64,
        "num_changed_lines": 148,
    }
    assert get_annotations(diff_cover_json) == [
        Annotation(
            path="github/annotate_missing_test_coverage.py",
            start_line=14,
            end_line=21,
            annotation_level="failure",
            message=f"Missing test coverage for 8 lines.{TIP_TEXT}",
        ),
        Annotation(
            path="github/annotate_missing_test_coverage.py",
            start_line=62,
            end_line=67,
            annotation_level="failure",
            message=f"Missing test coverage for 6 lines.{TIP_TEXT}",
        ),
        Annotation(
            path="github/annotate_missing_test_coverage.py",
            start_line=76,
            end_line=76,
            annotation_level="failure",
            message=f"Missing test coverage for 1 line.{TIP_TEXT}",
        ),
        Annotation(
            path="github/annotate_missing_test_coverage.py",
            start_line=80,
            end_line=80,
            annotation_level="failure",
            message=f"Missing test coverage for 1 line.{TIP_TEXT}",
        ),
        Annotation(
            path="github/annotate_missing_test_coverage.py",
            start_line=90,
            end_line=90,
            annotation_level="failure",
            message=f"Missing test coverage for 1 line.{TIP_TEXT}",
        ),
    ]


MARKDOWN_TEST_REPORT = """## Python Test Coverage Report

# Diff Coverage
## Diff: HEAD^...HEAD, staged and unstaged changes

- github/annotate_missing_test_coverage&#46;py (73.8%): Missing lines 14-21,78-82,86-87,97

## Summary

- **Total**: 61 lines
- **Missing**: 16 lines
- **Coverage**: 73%



## github/annotate_missing_test_coverage&#46;py

Lines 10-25

```python
from github.Repository import Repository
"""


def test_get_summary() -> None:
    assert (
        get_summary(MARKDOWN_TEST_REPORT)
        == """## Python Test Coverage Report

# Diff Coverage
## Diff: HEAD^...HEAD, staged and unstaged changes

- github/annotate_missing_test_coverage&#46;py (73.8%): Missing lines 14-21,78-82,86-87,97

## Summary

- **Total**: 61 lines
- **Missing**: 16 lines
- **Coverage**: 73%"""
    )


@patch("bar_raiser.checks.annotate_diff_cover.get_github_repo")
@patch("bar_raiser.checks.annotate_diff_cover.get_head_sha")
@patch(
    "bar_raiser.checks.annotate_diff_cover.load",
    side_effect=[{"total_percent_covered": 70, "src_stats": {}}],
)
@patch(
    "bar_raiser.checks.annotate_diff_cover.Path.open",
    new_callable=mock_open,
    read_data='{"total_percent_covered": 70, "src_stats": {}}',
)
@patch(
    "bar_raiser.checks.annotate_diff_cover.create_arg_parser_with_slack_dm_on_failure"
)
def test_main(
    mock_create_arg_parser: Mock,
    mock_open: Mock,
    mock_load: Mock,
    mock_get_head_sha: Mock,
    mock_get_github_repo: Mock,
) -> None:
    mock_parser = mock_create_arg_parser.return_value
    mock_parser.parse_args.return_value = argparse.Namespace(
        diff_cover_json_report=Path("diff_cover.json"),
        diff_cover_markdown_report=Path("diff_cover.md"),
        slack_dm_on_failure=None,
    )

    with patch(
        "bar_raiser.checks.annotate_diff_cover.create_check_run"
    ) as mock_create_check_run:
        main()

        mock_create_check_run.assert_called_once()
        mock_get_github_repo.assert_called_once()
        mock_get_head_sha.assert_called_once()
        mock_load.assert_called_once()
        assert mock_open.call_count == 2



from __future__ import annotations

from itertools import groupby
from json import load
from operator import itemgetter
from pathlib import Path
from typing import Literal, TypedDict

from bar_raiser.utils.check import create_arg_parser_with_slack_dm_on_failure
from bar_raiser.utils.github import (
    Annotation,
    create_check_run,
    get_github_repo,
    get_head_sha,
)
from bar_raiser.utils.slack import dm_on_check_failure

TIP_TEXT = " Please add tests for these lines.\nIf you believe there is a good reason to skip it, please click the '+' button to add an inline comment on this pull request to let the reviewers know."


def get_ranges(data: list[int]) -> list[tuple[int, int]]:
    ranges: list[tuple[int, int]] = []
    for _k, g in groupby(enumerate(data), lambda x: x[0] - x[1]):
        group = map(itemgetter(1), g)
        group = list(map(int, group))
        ranges.append((group[0], group[-1]))
    return ranges


class DiffCoverStat(TypedDict):
    violation_lines: list[int]


class DiffCoverJson(TypedDict):
    src_stats: dict[str, DiffCoverStat]
    total_percent_covered: int
    total_num_lines: int
    total_num_violations: int
    num_changed_lines: int


def get_annotations(diff_cover_json: DiffCoverJson) -> list[Annotation]:
    annotations: list[Annotation] = []
    for path, stats in diff_cover_json["src_stats"].items():
        for start, end in get_ranges(stats["violation_lines"]):
            lines = end - start + 1
            lines_msg = "1 line" if lines == 1 else f"{lines} lines"
            annotations.append(
                Annotation(
                    path=path,
                    start_line=start,
                    end_line=end,
                    annotation_level="failure",
                    message=f"Missing test coverage for {lines_msg}.{TIP_TEXT}",
                )
            )
    return annotations


def get_conclusion(coverage: float) -> Literal["success", "action_required"]:
    if coverage >= 75:
        return "success"
    return "action_required"


def get_summary(markdown_report: str) -> str:
    lines = markdown_report.split("\n")
    for i in range(len(lines)):
        if lines[i : i + 3] == ["", "", ""]:
            return "\n".join(lines[:i])
    return markdown_report


CHECK_NAME = "python-diff-cover-report"


def main():
    parser = create_arg_parser_with_slack_dm_on_failure()
    parser.add_argument(
        "diff_cover_json_report",
        type=Path,
        help="Path to the diff-cover generated json report",
    )
    parser.add_argument(
        "diff_cover_markdown_report",
        type=Path,
        help="Path to the diff-cover generated markdown report",
    )
    args = parser.parse_args()
    diff_cover_json: DiffCoverJson = load(
        args.diff_cover_json_report.open(encoding="utf-8")
    )
    repo = get_github_repo()
    checks = create_check_run(
        repo=repo,
        name=CHECK_NAME,
        head_sha=get_head_sha(),
        conclusion=get_conclusion(diff_cover_json["total_percent_covered"]),
        title="Python Test Coverage",
        summary=get_summary(
            args.diff_cover_markdown_report.read_text(encoding="utf-8")
        ),
        annotations=get_annotations(diff_cover_json),
        actions=[],
    )
    if args.slack_dm_on_failure:
        dm_on_check_failure(checks, args.slack_dm_on_failure)


if __name__ == "__main__":
    main()



from __future__ import annotations

from json import loads
from logging import getLogger
from os import environ
from typing import TYPE_CHECKING

from slack.web.client import WebClient

from bar_raiser.utils.github import get_pull_request

if TYPE_CHECKING:
    from pathlib import Path

    from github.CheckRun import CheckRun
    from slack.web.slack_response import SlackResponse

logger = getLogger(__name__)


def post_a_slack_message(
    channel: str, text: str, icon_url: str | None = None, username: str | None = None
):
    client = WebClient(token=environ["SLACK_BOT_TOKEN"])
    client.chat_postMessage(  # pyright: ignore[reportUnknownMemberType]
        channel=channel, text=text, icon_url=icon_url, username=username
    )


def get_slack_channel_from_mapping_path(key: str, mapping_path: Path) -> str | None:
    return loads(mapping_path.read_text()).get(key, None)


def dm_on_check_failure(
    checks: list[CheckRun], mapping_path: Path, message: str | None = None
):
    pull = get_pull_request()

    if pull and not pull.draft:
        user_id = get_slack_channel_from_mapping_path(pull.user.login, mapping_path)
        if user_id:
            failed_checks = [
                check
                for check in checks
                if check.conclusion in {"action_required", "failure"}
            ]
            if failed_checks:
                check_urls = " ".join(check.html_url for check in failed_checks)
                base_message = f"Github check `{failed_checks[0].name}` failed on <{pull.html_url}|PR-{pull.number}>: {check_urls}"
                slack_message = (
                    f"{message}\n\n{base_message}" if message else base_message
                )
                post_a_slack_message(
                    channel=user_id,
                    text=slack_message,
                )
    else:
        logger.info("No pull request found.")


def get_slack_user_icon_url_and_username(
    user_id: str,
) -> tuple[str, str] | tuple[None, None]:
    client = WebClient(token=environ["SLACK_BOT_TOKEN"])
    response: SlackResponse = client.users_info(user=user_id)  # pyright: ignore[reportUnknownMemberType,reportAssignmentType]
    try:
        if response["ok"]:
            user_info = response["user"]  # pyright: ignore[reportUnknownVariableType]
            if user_info:
                return user_info["profile"]["image_72"], user_info["real_name"]  # pyright: ignore[reportUnknownVariableType]
    except Exception:
        logger.exception(f"Error getting slack user info for {user_id}.")
    return None, None



from __future__ import annotations

from argparse import ArgumentParser
from dataclasses import dataclass
from pathlib import Path
from re import match

from bar_raiser.utils.github import Action, Annotation, Autofixes


@dataclass(frozen=True)
class CheckPattern:
    regex: str
    message: str | None = None
    line: int | None = None
    should_create_annotation: bool = True
    is_autofixable: bool = False


def parse_line(
    pattern: CheckPattern, line: str, repo: Path
) -> tuple[Annotation | None, bool]:
    matched = match(pattern.regex, line)
    is_autofixable = pattern.is_autofixable
    if matched:
        line_num = int(matched.group("line")) if pattern.line is None else pattern.line
        return (
            Annotation(
                path=str(Path(Path.cwd() / matched.group("path")).relative_to(repo)),
                start_line=line_num,
                end_line=line_num,
                annotation_level="failure",
                message=matched.group("message")
                if pattern.message is None
                else pattern.message,
            ),
            is_autofixable,
        )
    return (None, is_autofixable)


def get_annotations_and_actions(
    repo_dir: Path, ruff_output: str, patterns: list[CheckPattern], autofix: Autofixes
) -> tuple[list[Annotation], Action | None]:
    annotations: list[Annotation] = []
    is_autofix_available = False
    for line in ruff_output.split("\n"):
        for pattern in patterns:
            if pattern.should_create_annotation:
                annotation, is_autofixable = parse_line(pattern, line, repo_dir)
                if annotation:
                    annotations.append(annotation)
                    if is_autofixable:
                        is_autofix_available = is_autofixable
            elif (
                is_autofix_available is False
                and pattern.is_autofixable
                and match(pattern.regex, line)
            ):
                is_autofix_available = True
    action = (
        Action(
            label="autofix",
            description="Click to auto-push an autofix commit",
            identifier=str(autofix),
        )
        if is_autofix_available
        else None
    )
    return annotations, action


def create_arg_parser_with_slack_dm_on_failure() -> ArgumentParser:
    parser = ArgumentParser(
        description="Run checks and optionally send Slack DMs on failure."
    )
    parser.add_argument(
        "--slack-dm-on-failure",
        type=Path,
        help="Path to a JSON file containing a mapping from GitHub login to Slack user ID.",
        default=None,
    )
    return parser



from __future__ import annotations

import sys
from json import dumps
from pathlib import Path
from unittest.mock import patch

from bar_raiser.checks.annotate_pytest import PytestReportJson, get_annotations, main

REPO_DIR = "/home/user/bar_raiser"
WORKING_DIR = "/home/user/bar_raiser/subfolder"

pytest_report_json: PytestReportJson = {
    "root": WORKING_DIR,
    "summary": {
        "passed": 12,
        "failed": 1,
        "total": 13,
    },
    "tests": [
        {
            "nodeid": "test_rules.py::NoAwaitInLoopRule::test_INVALID_0",
            "lineno": 223,
            "outcome": "passed",
            "call": {},
        },
        {
            "nodeid": "test_rules.py::NoAwaitInLoopRule::test_VALID_0",
            "lineno": 223,
            "outcome": "passed",
            "call": {},
        },
        {
            "nodeid": "github/test_utils.py::test_get_repo",
            "lineno": 8,
            "outcome": "failed",
            "call": {
                "longrepr": '@patch.dict(\n        environ,\n        {\n            "APP_ID": "_ID",\n            "PRIVATE_KEY": "_KEY",\n            "GITHUB_REPOSITORY_OWNER": "Greenbax",\n            "GITHUB_REPOSITORY": "Greenbax/evergreen",\n        },\n    )\n    def test_get_repo() -> None:\n>       with patch(\n            "fixit_linters.github.annotate_missing_test_coverage.GithubIntegration"\n        ), patch(\n            "fixit_linters.github.annotate_missing_test_coverage.Github"\n        ) as mock_github:\n\nfixit_linters/github/test_utils.py:19: \n_ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _ _\n\nself = <unittest.mock._patch object at 0x7f896fc95b10>\n\n    def get_original(self):\n        target = self.getter()\n        name = self.attribute\n    \n        original = DEFAULT\n        local = False\n    \n        try:\n            original = target.__dict__[name]\n        except (AttributeError, KeyError):\n            original = getattr(target, name, DEFAULT)\n        else:\n            local = True\n    \n        if name in _builtins and isinstance(target, ModuleType):\n            self.create = True\n    \n        if not self.create and original is DEFAULT:\n>           raise AttributeError(\n                "%s does not have the attribute %r" % (target, name)\n            )\nE           AttributeError: <module \'fixit_linters.github.annotate_missing_test_coverage\' from \'/home/admin/evergreen/website/fixit_linters/github/annotate_missing_test_coverage.py\'> does not have the attribute \'GithubIntegration\'\n\n../../.pyenv/versions/3.11.3/lib/python3.11/unittest/mock.py:1410: AttributeError'
            },
        },
        {
            "nodeid": "github/test_utils.py::test_create_check_run",
            "lineno": 27,
            "outcome": "passed",
            "call": {},
        },
    ],
}


def test_get_annotations() -> None:
    annotations = get_annotations(pytest_report_json, Path(REPO_DIR))
    assert len(annotations) == 1
    assert annotations[0]["path"] == "subfolder/github/test_utils.py"


def test_main() -> None:
    with (
        patch(
            "bar_raiser.checks.annotate_pytest.create_check_run"
        ) as mock_create_check_run,
        patch("bar_raiser.checks.annotate_pytest.get_git_repo") as mock_get_git_repo,
        patch("bar_raiser.checks.annotate_pytest.get_github_repo"),
        patch("bar_raiser.checks.annotate_pytest.get_head_sha", return_value="1"),
        patch(
            "bar_raiser.checks.annotate_pytest.dm_on_check_failure"
        ) as mock_dm_on_check_failure,
        patch.object(sys, "argv", ["annotate_pytest.py", "path/to/report.json"]),
        patch("pathlib.Path.read_text", return_value=dumps(pytest_report_json)),
    ):
        mock_get_git_repo.return_value.working_dir = Path(WORKING_DIR)
        main()
        assert len(mock_create_check_run.call_args_list) == 1
        kwargs = mock_create_check_run.call_args_list[0].kwargs
        assert kwargs["name"] == "python-pytest-report"
        assert kwargs["head_sha"] == "1"
        assert kwargs["conclusion"] == "action_required"
        assert kwargs["title"] == "Python Pytest Report"
        assert len(kwargs["annotations"]) == 1
        mock_dm_on_check_failure.assert_not_called()



from __future__ import annotations

from json import loads
from pathlib import Path
from typing import NotRequired, TypedDict

from bar_raiser.utils.check import create_arg_parser_with_slack_dm_on_failure
from bar_raiser.utils.github import (
    Annotation,
    create_check_run,
    get_git_repo,
    get_github_repo,
    get_head_sha,
)
from bar_raiser.utils.slack import dm_on_check_failure

CHECK_NAME = "python-pytest-report"


class Call(TypedDict):
    longrepr: NotRequired[str]


class Test(TypedDict):
    nodeid: str
    outcome: str
    lineno: int
    call: Call


class TestSummary(TypedDict):
    passed: NotRequired[int]
    failed: int
    total: int


class PytestReportJson(TypedDict):
    root: str
    summary: TestSummary
    tests: list[Test]


def get_annotations(
    pytest_report_json: PytestReportJson, git_root: Path
) -> list[Annotation]:
    annotations: list[Annotation] = []
    for test in pytest_report_json["tests"]:
        if test["outcome"] == "failed":
            full_path = Path(pytest_report_json["root"]).joinpath(
                test["nodeid"].split("::")[0]
            )
            annotations.append(
                Annotation(
                    path=str(full_path.relative_to(git_root)),
                    start_line=test["lineno"],
                    end_line=test["lineno"],
                    annotation_level="failure",
                    message=test["call"].get(
                        "longrepr", "Missing longrepr from pytest report."
                    ),
                )
            )
    return annotations


def get_summary(pytest_report_json: PytestReportJson) -> str:
    summary = pytest_report_json["summary"]
    return f"Passed: {summary.get('passed', 0)}, Failed: {summary.get('failed', 0)}, Total: {summary['total']}"


def main():
    parser = create_arg_parser_with_slack_dm_on_failure()
    parser.add_argument(
        "pytest_json_report",
        type=Path,
        help="Path to the pytest json report crated with --json-report option using pytest-json-report",
    )
    args = parser.parse_args()
    pytest_report_json: PytestReportJson = loads(args.pytest_json_report.read_text())
    annotations = get_annotations(pytest_report_json, Path(get_git_repo().working_dir))
    checks = create_check_run(
        repo=get_github_repo(),
        name=CHECK_NAME,
        head_sha=get_head_sha(),
        conclusion="action_required" if len(annotations) > 0 else "success",
        title="Python Pytest Report",
        summary=get_summary(pytest_report_json),
        annotations=annotations,
        actions=[],
    )
    if args.slack_dm_on_failure:
        dm_on_check_failure(checks, args.slack_dm_on_failure)


if __name__ == "__main__":
    main()



from __future__ import annotations

from logging import getLogger
from typing import TYPE_CHECKING

from bar_raiser.utils.check import create_arg_parser_with_slack_dm_on_failure
from bar_raiser.utils.github import (
    create_check_run,
    get_github_repo,
    get_head_sha,
    get_pull_request,
    initialize_logging,
)
from bar_raiser.utils.slack import dm_on_check_failure

if TYPE_CHECKING:
    from github.Commit import Commit
    from github.PaginatedList import PaginatedList

logger = getLogger(__name__)
CHECK_NAME = "avoid-merge-commits"


def contains_merge_commit(commits: PaginatedList[Commit]) -> bool:
    """
    Check if any commit in the list is a merge commit.
    """
    return any(len(commit.parents) > 1 for commit in commits)


def main():
    parser = create_arg_parser_with_slack_dm_on_failure()
    args = parser.parse_args()
    repo = get_github_repo()
    pull_request = get_pull_request()
    if not pull_request:
        logger.info("No pull request found.")
        return

    # Retrieve commits from the pull request
    commits = pull_request.get_commits()
    logger.info(f"Commits: {commits}")

    # Check for merge commits
    if contains_merge_commit(commits):
        conclusion = "action_required"
        title = "Merge Commits Detected"
        summary = (
            f"Your pull request #{pull_request.number} contains merge commits. "
            "Please rebase your commits onto the latest master branch to prevent potential linting errors."
        )
    else:
        conclusion = "success"
        title = "No Merge Commits"
        summary = f"Your pull request #{pull_request.number} does not contain any merge commits."

    # Create a check run
    checks = create_check_run(
        repo=repo,
        name=CHECK_NAME,
        head_sha=get_head_sha(),
        conclusion=conclusion,
        title=title,
        summary=summary,
        annotations=[],  # No annotations needed for this check
        actions=[],
    )

    message = "CAUTION: Please rebase your commits onto the latest master branch to prevent potential linting errors."

    if args.slack_dm_on_failure:
        dm_on_check_failure(checks, args.slack_dm_on_failure, message)
    else:
        logger.info("No slack_dm_on_failure configured.")


if __name__ == "__main__":
    initialize_logging()
    main()



from __future__ import annotations

from argparse import ArgumentParser
from pathlib import Path

from bar_raiser.utils.check import create_arg_parser_with_slack_dm_on_failure


def test_create_arg_parser_with_slack_dm_on_failure():
    parser = create_arg_parser_with_slack_dm_on_failure()
    assert isinstance(parser, ArgumentParser)
    args = parser.parse_args(["--slack-dm-on-failure", "path/to/file.json"])
    assert args.slack_dm_on_failure == Path("path/to/file.json")



from __future__ import annotations

from argparse import ArgumentParser, Namespace
from typing import TYPE_CHECKING
from unittest.mock import MagicMock, create_autospec, patch

from github.Commit import Commit
from github.PaginatedList import PaginatedList
from github.PullRequest import PullRequest
from github.Repository import Repository

from bar_raiser.checks.annotate_merge_commits import (
    CHECK_NAME,
    contains_merge_commit,
    main,
)

if TYPE_CHECKING:
    from collections.abc import Iterator


class CustomPaginatedList(PaginatedList[Commit]):
    def __init__(self, commits: list[Commit]) -> None:
        self._commits = commits

    def __iter__(self) -> Iterator[Commit]:
        return iter(self._commits)


def test_contains_merge_commit_true() -> None:
    commit1 = MagicMock(spec=Commit)
    commit1.parents = [MagicMock(), MagicMock()]  # Merge commit (2 parents)
    commit2 = MagicMock(spec=Commit)
    commit2.parents = [MagicMock()]  # Regular commit (1 parent)
    commits = CustomPaginatedList([commit1, commit2])

    result = contains_merge_commit(commits)

    assert result is True


def test_contains_merge_commit_false() -> None:
    commit1 = MagicMock(spec=Commit)
    commit1.parents = [MagicMock()]
    commit2 = MagicMock(spec=Commit)
    commit2.parents = [MagicMock()]
    commits = CustomPaginatedList([commit1, commit2])

    result = contains_merge_commit(commits)

    assert result is False


@patch(
    "bar_raiser.checks.annotate_merge_commits.create_arg_parser_with_slack_dm_on_failure"
)
@patch("bar_raiser.checks.annotate_merge_commits.get_github_repo")
@patch("bar_raiser.checks.annotate_merge_commits.get_pull_request")
@patch("bar_raiser.checks.annotate_merge_commits.get_head_sha")
@patch("bar_raiser.checks.annotate_merge_commits.create_check_run")
@patch("bar_raiser.checks.annotate_merge_commits.dm_on_check_failure")
def test_amc_with_merge_commits(  # noqa: PLR0917
    mock_dm_on_check_failure: MagicMock,
    mock_create_check_run: MagicMock,
    mock_get_head_sha: MagicMock,
    mock_get_pull_request: MagicMock,
    mock_get_github_repo: MagicMock,
    mock_create_arg_parser: MagicMock,
) -> None:
    mock_args = MagicMock(spec=Namespace)
    mock_args.slack_dm_on_failure = mapping_path = "user_mapping.json"

    mock_parser = create_autospec(spec=ArgumentParser)
    mock_parser.parse_args.return_value = mock_args
    mock_create_arg_parser.return_value = mock_parser

    mock_repo = MagicMock(spec=Repository)
    mock_get_github_repo.return_value = mock_repo

    mock_pr = create_autospec(spec=PullRequest)
    pr_num = 123
    mock_pr.number = pr_num

    commit1 = MagicMock(spec=Commit)
    commit1.parents = [MagicMock(), MagicMock()]  # Merge commit

    mock_commits = CustomPaginatedList([commit1])
    mock_pr.get_commits.return_value = mock_commits
    mock_get_pull_request.return_value = mock_pr

    mock_head_sha = "abc123"
    mock_get_head_sha.return_value = mock_head_sha

    mock_check_run = MagicMock()
    mock_create_check_run.return_value = mock_check_run

    main()

    mock_create_check_run.assert_called_once_with(
        repo=mock_repo,
        name=CHECK_NAME,
        head_sha=mock_head_sha,
        conclusion="action_required",
        title="Merge Commits Detected",
        summary=(
            f"Your pull request #{pr_num} contains merge commits. "
            "Please rebase your commits onto the latest master branch to prevent potential linting errors."
        ),
        annotations=[],
        actions=[],
    )

    mock_dm_on_check_failure.assert_called_once_with(
        mock_check_run,
        mapping_path,
        "CAUTION: Please rebase your commits onto the latest master branch to prevent potential linting errors.",
    )


@patch(
    "bar_raiser.checks.annotate_merge_commits.create_arg_parser_with_slack_dm_on_failure"
)
@patch("bar_raiser.checks.annotate_merge_commits.get_github_repo")
@patch("bar_raiser.checks.annotate_merge_commits.get_pull_request")
@patch("bar_raiser.checks.annotate_merge_commits.get_head_sha")
@patch("bar_raiser.checks.annotate_merge_commits.create_check_run")
@patch("bar_raiser.checks.annotate_merge_commits.dm_on_check_failure")
def test_amc_without_merge_commits(  # noqa: PLR0917
    mock_dm_on_check_failure: MagicMock,
    mock_create_check_run: MagicMock,
    mock_get_head_sha: MagicMock,
    mock_get_pull_request: MagicMock,
    mock_get_github_repo: MagicMock,
    mock_create_arg_parser: MagicMock,
) -> None:
    mock_args = MagicMock(spec=Namespace)
    mock_args.slack_dm_on_failure = None

    mock_parser = create_autospec(spec=ArgumentParser)
    mock_parser.parse_args.return_value = mock_args
    mock_create_arg_parser.return_value = mock_parser

    mock_repo = MagicMock(spec=Repository)
    mock_get_github_repo.return_value = mock_repo

    mock_pr = create_autospec(spec=PullRequest)
    pr_num = 123
    mock_pr.number = pr_num

    commit1 = MagicMock(spec=Commit)
    commit1.parents = [MagicMock()]
    mock_commits = CustomPaginatedList([commit1])
    mock_pr.get_commits.return_value = mock_commits
    mock_get_pull_request.return_value = mock_pr

    mock_head_sha = "abc123"
    mock_get_head_sha.return_value = mock_head_sha

    main()

    mock_create_check_run.assert_called_once_with(
        repo=mock_repo,
        name=CHECK_NAME,
        head_sha=mock_head_sha,
        conclusion="success",
        title="No Merge Commits",
        summary=f"Your pull request #{pr_num} does not contain any merge commits.",
        annotations=[],
        actions=[],
    )

    mock_dm_on_check_failure.assert_not_called()


@patch(
    "bar_raiser.checks.annotate_merge_commits.create_arg_parser_with_slack_dm_on_failure"
)
@patch("bar_raiser.checks.annotate_merge_commits.get_github_repo")
@patch("bar_raiser.checks.annotate_merge_commits.get_pull_request")
@patch("bar_raiser.checks.annotate_merge_commits.logger.info")
def test_amc_no_pull_request(
    mock_logger: MagicMock,
    mock_get_pull_request: MagicMock,
    mock_get_github_repo: MagicMock,
    mock_create_arg_parser: MagicMock,
) -> None:
    mock_args = MagicMock(spec=Namespace)

    mock_parser = create_autospec(spec=ArgumentParser)
    mock_parser.parse_args.return_value = mock_args
    mock_create_arg_parser.return_value = mock_parser

    mock_repo = MagicMock(spec=Repository)
    mock_get_github_repo.return_value = mock_repo
    mock_get_pull_request.return_value = None

    main()

    mock_logger.assert_called_once_with("No pull request found.")



from __future__ import annotations

from os import environ
from unittest.mock import MagicMock, mock_open, patch

from git import Diff
from github.File import File
from github.IssueComment import IssueComment
from github.NamedUser import NamedUser
from github.PullRequest import PullRequest
from github.Repository import Repository

from bar_raiser.utils.github import (
    Annotation,
    commit_changes,
    create_a_pull_request,
    create_check_run,
    get_github_repo,
    get_pull_request,
    get_updated_paths,
    has_previous_issue_comment,
    run_codemod_and_commit_changes,
)

TEST_ORG = "ZipHQ"
TEST_REPO = f"{TEST_ORG}/bar-raiser"


@patch.dict(
    environ,
    {
        "APP_ID": "_ID",
        "PRIVATE_KEY": "_KEY",
        "GITHUB_REPOSITORY_OWNER": TEST_ORG,
        "GITHUB_REPOSITORY": TEST_REPO,
    },
)
def test_get_repo() -> None:  # touch
    with (
        patch("bar_raiser.utils.github.GithubIntegration"),
        patch("bar_raiser.utils.github.Github") as mock_github,
    ):
        get_github_repo()
        mock_github.return_value.get_repo.assert_called_with(TEST_REPO)


def test_create_check_run_with_pagination() -> None:
    mock_repo = MagicMock(spec=Repository)
    create_check_run(
        repo=mock_repo,
        name="",
        head_sha="",
        conclusion="action_required",
        title="",
        summary="",
        annotations=[
            Annotation(
                path="", start_line=1, end_line=1, annotation_level="", message=""
            )
            for i in range(75)
        ],
        actions=[],
    )
    assert mock_repo.create_check_run.call_count == 2


def test_create_check_run_with_no_annotations() -> None:
    mock_repo = MagicMock(spec=Repository)
    create_check_run(
        repo=mock_repo,
        name="",
        head_sha="",
        conclusion="success",
        title="",
        summary="",
        annotations=[],
        actions=[],
    )
    assert mock_repo.create_check_run.call_count == 1


def test_commit_changes() -> None:
    mock_repo = MagicMock(spec=Repository)
    # fmt: off
    mock_repo.create_git_blob.return_value = MagicMock(
        sha="a_sha"
    )
    # fmt: on
    with patch("bar_raiser.utils.github.open", mock_open(read_data="")):
        commit_changes(mock_repo, "a_branch", "a_sha", ["a.py"], "a_commit_message")
    assert mock_repo.create_git_commit.call_count == 1
    mock_repo.reset_mock()

    with patch("bar_raiser.utils.github.open", mock_open(read_data="")):
        commit_changes(
            mock_repo,
            "a_branch",
            "a_sha",
            [f"a_{i}.py" for i in range(201)],
            "a_commit_message",
        )
    assert mock_repo.create_git_commit.call_count == 2


def test_run_codemod_and_commit_changes() -> None:
    mock_repo = MagicMock(spec=Repository)
    with (
        patch("bar_raiser.utils.github.check_output") as mock_check_output,
        patch("bar_raiser.utils.github.Repo") as mock_git_repo,
        patch("bar_raiser.utils.github.commit_changes") as mock_commit_changes,
    ):
        mock_git_repo.return_value.index.diff.return_value = []
        run_codemod_and_commit_changes(
            mock_repo, 123, [["black"]], "Apply Black Format", run_on_updated_paths=True
        )
        mock_check_output.assert_called_with(["black"])
        mock_commit_changes.assert_not_called()
        mock_pull = mock_repo.get_pull.return_value
        mock_pull.create_issue_comment.assert_called_once_with(
            "No updated paths to commit."
        )

        mock_git_repo.return_value.index.diff.return_value = [
            MagicMock(spec=Diff, b_path="a.py")
        ]
        mock_pull.get_files.return_value = [
            MagicMock(spec=str, status="added", filename="a.py")
        ]
        mock_pull.create_issue_comment.reset_mock()
        run_codemod_and_commit_changes(
            mock_repo, 123, [["black"]], "Apply Black Format", run_on_updated_paths=True
        )
        mock_commit_changes.assert_called_once()
        mock_pull.create_issue_comment.assert_not_called()


def test_get_updated_paths() -> None:
    mock_pull = MagicMock(spec=PullRequest)
    mock_pull.get_files.return_value = [
        MagicMock(spec=File, filename="a.py", status="added"),
        MagicMock(spec=File, filename="b.py", status="removed"),
    ]
    get_updated_paths(mock_pull)


@patch.dict(
    environ,
    {
        "GITHUB_SERVER_URL": "https://github.com",
        "GITHUB_REPOSITORY": TEST_REPO,
        "GITHUB_RUN_ID": "12345",
    },
)
def test_create_a_pull_request() -> None:
    with (
        patch("bar_raiser.utils.github.Repo") as mock_git_repo,
        patch("bar_raiser.utils.github.commit_changes") as mock_commit_changes,
    ):
        mock_git_repo.return_value.index.diff.return_value = [
            MagicMock(spec=Diff, b_path="a.py")
        ]
        mock_github_repo = MagicMock(spec=Repository)
        create_a_pull_request(
            mock_github_repo,
            "python3 -m libcst.tool codemod convert_to_async.AsyncFunctionTransformer --target-qualified-name pre_call_handler integrations/erp_client_metrics_wrapper.py",
            "jimmy",
            extra_body="some more context",
        )
        mock_commit_changes.assert_called_once()
        mock_github_repo.create_pull.assert_called_once()
        call = mock_github_repo.create_pull.call_args
        assert call.kwargs["title"].startswith(
            "Codemod convert_to_async.AsyncFunctionTransformer by jimmy at "
        )
        assert call.kwargs["head"].startswith(
            "refs/heads/codemod-convert_to_async.AsyncFunctionTransformer-"
        )
        assert call.kwargs["body"].endswith("some more context")


def test_has_previous_issue_comment() -> None:
    assert (
        has_previous_issue_comment(
            MagicMock(spec=PullRequest), "jimmy", "net new tech debt"
        )
        is False
    )
    pull = MagicMock(
        spec=PullRequest,
        get_issue_comments=lambda: [
            MagicMock(
                spec=IssueComment,
                user=MagicMock(spec=NamedUser, login="jimmy"),
                body="You introduced some net new tech debt",
            )
        ],
    )
    assert has_previous_issue_comment(pull, "jimmy", "net new tech debt") is True


def test_get_pull_request() -> None:
    with patch("bar_raiser.utils.github.get_github_repo"):
        assert get_pull_request() is None



from __future__ import annotations

from json import loads
from logging import getLogger
from pathlib import Path
from subprocess import CalledProcessError, check_output
from sys import exit

from bar_raiser.utils.check import create_arg_parser_with_slack_dm_on_failure
from bar_raiser.utils.github import (
    Action,
    Annotation,
    Autofixes,
    create_check_run,
    get_git_repo,
    get_github_repo,
    get_head_sha,
    initialize_logging,
)
from bar_raiser.utils.slack import dm_on_check_failure

logger = getLogger(__name__)

CHECK_NAME = "python-pyright-report"


def get_annotations_and_actions_for_pyright_check(
    working_dir: Path, pyright_output_json: str
) -> tuple[list[Annotation], list[Action]]:
    annotations: list[Annotation] = []
    data = loads(pyright_output_json)
    action_ids: set[str] = set()
    for error in data["generalDiagnostics"]:
        rule = f" [{error['rule']}]" if "rule" in error else ""
        annotations.append(
            Annotation(
                path=str(Path(error["file"]).relative_to(working_dir)),
                start_line=error["range"]["start"]["line"]
                + 1,  # pyright uses 0-based line numbers
                end_line=error["range"]["end"]["line"] + 1,
                annotation_level="failure",
                message=error["message"] + rule,
            )
        )
        if 'Unnecessary "# pyright: ignore"' in error["message"]:
            action_ids.add(Autofixes.PYRIGHT_IGNORES.value)
    actions = [
        Action(
            label="autofix",
            description="Click to auto-push an autofix commit",
            identifier=action_id,
        )
        for action_id in action_ids
    ]
    return annotations, actions


def main() -> None:
    initialize_logging()
    args = create_arg_parser_with_slack_dm_on_failure().parse_args()
    git_repo = get_git_repo()
    annotations: list[Annotation] = []
    actions: list[Action] = []
    return_code = -1
    try:
        output = check_output(
            [
                "pyright",
                "--outputjson",
            ],
        )
        pyright_output_json = output.decode("utf-8")
        return_code = 0
    except CalledProcessError as e:
        pyright_output_json = e.output.decode("utf-8")
        annotations, actions = get_annotations_and_actions_for_pyright_check(
            Path(git_repo.working_dir), pyright_output_json
        )
        return_code = e.returncode

    summary = f"Pyright found {len(annotations)} errors."
    if len(actions) > 0:
        summary += "Autofix is available. Simply click :point_up_2: the above `autofix` button to apply.\n"
        summary += "After the autofix, if you plan to continue developing, run `git pull --rebase` to fetch the changes in your working directory.\n\n"

    logger.info(summary)
    for annotation in annotations:
        logger.info(
            f"{annotation['path']}:{annotation['start_line']}:{annotation['message']}"
        )
    checks = create_check_run(
        repo=get_github_repo(),
        name=CHECK_NAME,
        head_sha=get_head_sha(),
        conclusion="success" if len(annotations) == 0 else "action_required",
        title="Python Pyright Type Checker",
        summary=summary,
        annotations=annotations,
        actions=actions,
    )
    if args.slack_dm_on_failure:
        dm_on_check_failure(checks, args.slack_dm_on_failure)

    exit(return_code)


if __name__ == "__main__":
    main()



from __future__ import annotations

from logging import getLogger
from pathlib import Path
from subprocess import STDOUT, CalledProcessError, check_output
from sys import exit
from typing import TYPE_CHECKING

from bar_raiser.utils.check import (
    CheckPattern,
    create_arg_parser_with_slack_dm_on_failure,
    get_annotations_and_actions,
)
from bar_raiser.utils.github import (
    Action,
    Annotation,
    Autofixes,
    create_check_run,
    get_git_repo,
    get_github_repo,
    get_head_sha,
    initialize_logging,
)
from bar_raiser.utils.slack import dm_on_check_failure

if TYPE_CHECKING:
    from collections.abc import Callable

logger = getLogger(__name__)


CHECK_NAME = "python-ruff-report"
WOULD_REFORMAT_PATTERN = CheckPattern(
    regex="^Would reformat: (?P<path>.*)$",
    message="Ruff would reformat this file.",
    line=1,
    is_autofixable=True,
)
CANNOT_FORMAT_PATTERN = CheckPattern(
    regex="^error: Failed to format (?P<path>[^:]+): (?P<message>.*), offset: (?P<line>[0-9]+), .*$"
)
RUFF_FORMAT_PATTERNS = [
    WOULD_REFORMAT_PATTERN,
    CANNOT_FORMAT_PATTERN,
]
CHECK_ERROR_PATTERN = CheckPattern(
    regex="^(?P<path>[^:]+):(?P<line>[0-9]+):[0-9]+: (?P<message>.*)$"
)
CANNOT_PARSE_PATTERN = CheckPattern(
    regex="^error: Failed to parse (?P<path>[^:]+):(?P<line>[0-9]+):[0-9]+: (?P<message>.*)$"
)
FIXABLE_PATTERN = CheckPattern(
    regex="^.* fixable with the `--fix` option.$",
    should_create_annotation=False,
    is_autofixable=True,
)
RUFF_CHECK_PATTERNS = [CANNOT_PARSE_PATTERN, CHECK_ERROR_PATTERN, FIXABLE_PATTERN]


def main() -> None:
    initialize_logging()
    args = create_arg_parser_with_slack_dm_on_failure().parse_args()
    git_repo = get_git_repo()
    annotations: list[Annotation] = []
    actions: list[Action] = []
    cmds_parsers: list[
        tuple[list[str], Callable[[Path, str], tuple[list[Annotation], Action | None]]]
    ] = [
        (
            ["ruff", "format", "--check", "."],
            lambda wd, output: get_annotations_and_actions(
                wd,
                output,
                [WOULD_REFORMAT_PATTERN, CANNOT_FORMAT_PATTERN],
                Autofixes.RUFF,
            ),
        ),
        (
            ["ruff", "check", "."],
            lambda wd, output: get_annotations_and_actions(
                wd,
                output,
                [CHECK_ERROR_PATTERN, CANNOT_PARSE_PATTERN, FIXABLE_PATTERN],
                Autofixes.RUFF,
            ),
        ),
    ]
    RETURN_CODE_NOT_SET = -100
    return_code = RETURN_CODE_NOT_SET
    for cmds, parser in cmds_parsers:
        try:
            output = check_output(cmds, stderr=STDOUT)
            ruff_output = output.decode("utf-8")
            if return_code == RETURN_CODE_NOT_SET:
                return_code = 0
        except CalledProcessError as e:
            ruff_output = e.output.decode("utf-8")
            return_code = e.returncode
            new_annotations, action = parser(Path(git_repo.working_dir), ruff_output)
            annotations.extend(new_annotations)
            if len(actions) == 0 and action is not None:
                actions.append(action)

        logger.info(ruff_output)

    summary = f"Ruff found {len(annotations)} errors."
    if len(actions) > 0:
        summary += "Autofix is available. Simply click :point_up_2: the above `autofix` button to apply.\n"
        summary += "After the autofix, if you plan to continue developing, run `git pull --rebase` to fetch the changes in your working directory.\n\n"
        summary += "To fix format errors manually in your working directory, run: `ruff format .`"
        summary += "To fix check errors manually in your working directory, run: `ruff check --fix .`"

    checks = create_check_run(
        repo=get_github_repo(),
        name=CHECK_NAME,
        head_sha=get_head_sha(),
        conclusion="success" if len(annotations) == 0 else "action_required",
        title="Python Ruff formatter and linter",
        summary=summary,
        annotations=annotations,
        actions=list(actions),
    )
    if len(actions) > 0:
        logger.info(
            f"Autofix is available. Simply click the autofix button on the following page to apply: {' '.join([check.html_url for check in checks])}"
        )
        logger.info(
            "After the autofix, if you plan to continue developing, run `git pull --rebase` to fetch the changes in your working directory."
        )
        logger.info(
            "To fix format errors manually in your working directory, run: `ruff format .`"
        )
        logger.info(
            "To fix check errors manually in your working directory, run: `ruff check --fix .`"
        )

    if args.slack_dm_on_failure:
        dm_on_check_failure(checks, args.slack_dm_on_failure)

    exit(return_code)


if __name__ == "__main__":
    main()



from __future__ import annotations

import json
from os import environ
from typing import TYPE_CHECKING
from unittest.mock import MagicMock, patch

from github.CheckRun import CheckRun
from github.PullRequest import PullRequest

from bar_raiser.utils.slack import dm_on_check_failure, post_a_slack_message

if TYPE_CHECKING:
    from pathlib import Path


@patch.dict(environ, {"SLACK_BOT_TOKEN": "xxx"})
def test_post_a_slack_message() -> None:
    CHANNEL = "C06V783RYAA"
    with patch("bar_raiser.utils.slack.WebClient") as mock_web_client:
        post_a_slack_message(CHANNEL, "test message")
        mock_web_client.return_value.chat_postMessage.assert_called_with(
            channel=CHANNEL, icon_url=None, text="test message", username=None
        )


def test_dm_on_check_failure(tmp_path: Path):
    # Create a dummy mapping file
    mapping_file = tmp_path / "user_mapping.json"
    channel = "U15NC768RK1"
    mapping_data = {"jimmy-lai-zip": channel}  # Example mapping
    with open(mapping_file, "w", encoding="utf-8") as f:
        json.dump(mapping_data, f)

    with (
        patch(
            "bar_raiser.utils.slack.get_pull_request",
            return_value=MagicMock(
                spec=PullRequest,
                draft=False,
                user=MagicMock(login="jimmy-lai-zip"),
                html_url="github.com/pull/1",
                number=1,
            ),
        ),
        patch(
            "bar_raiser.utils.slack.post_a_slack_message"
        ) as mock_post_a_slack_message,
    ):
        checks: list[CheckRun] = [
            MagicMock(
                spec=CheckRun,
                html_url="github.com/check/1",
                conclusion="action_required",
            ),
            MagicMock(
                spec=CheckRun,
                html_url="github.com/check/2",
                conclusion="success",
            ),
        ]
        checks[0].name = "check1"  # pyright: ignore[reportAttributeAccessIssue]
        checks[1].name = "check2"  # pyright: ignore[reportAttributeAccessIssue]
        dm_on_check_failure(checks, mapping_file)
        mock_post_a_slack_message.assert_called_once_with(
            channel=channel,
            text="Github check `check1` failed on <github.com/pull/1|PR-1>: github.com/check/1",
        )
